"""
zwiki mailin - post an incoming email message to a zwiki page

This is intended to be set up as an external method for receiving mail
from a mailer alias, procmail recipe or script via something like
| curl -n -F 'msg=<-' http://mysite/mywikifolder/mailin
The mailin method expects at least one argument, an RFC822 message,
which will be formatted as a wiki comment and appended to the
appropriate page.  Here are the default delivery rules:

- if called in a page context, always use that page. Otherwise,

- if a recipient of the form
  "[something](wiki|tracker|bugs|mailin)@virtualhost" is found where
  virtualhost matches an existing VirtualHostMonster entry, then in the
  corresponding folder, otherwise in the current folder, use:

   (disabled) 1. a wiki page name before @ in a recipient header
   2. or the first wiki page name appearing in the subject
   3. or the folder's default_page property
   4. or the supplied defaultpage argument
   5. or the hardcoded DEFAULTPAGE below

- if a non-existent page is named in the subject, create it, and add a
  link on the default page

- if the trackerissue flag is passed or any of the recipients contains
  "(tracker)" or there was a match for "[something]tracker@virtualhost",
  create a tracker issue page instead of posting a comment

- unless subscribersonly=0 or the folder's posting_policy property is
  'open': if the sender is not subscribed anywhere in the wiki, bounce the
  message

- if the message appears to have been mailed out by zwiki, silently
  discard it

todo:
friendly bounce messages
size limits
"""

from types import *
import re, string 
import rfc822, mimetools, multifile, StringIO
from Products.ZWiki.Regexps import wikiname1,wikiname2,bracketedexpr

DEFAULTPAGE = 'FrontPage'
#WIKINAMEEXP = r'(%s|%s|%s)' % (wikiname1,wikiname2,bracketedexpr)
WIKINAMEEXP = r'(%s|%s)' % (wikiname1,wikiname2)
RECOGNIZEDADDREXP = r'(wiki|tracker|bugs|mailin)@'

class IncomingMessage:
    """
    an incoming wikimail message

    responsibilities: parse an rfc822 and expose the parts of interest
    """
    def __init__(self,message):
        self.raw = message
        self.stream = StringIO.StringIO(self.raw)
        self.msg = mimetools.Message(self.stream)
        self.date = self.msg['Date']
        self.subject = (self.msg.has_key('Subject') and self.msg['Subject']) or ''
        self.sender = self.msg['From']
        self.senderEmail = rfc822.parseaddr(self.sender)[1]
        self.recipients = []
        for hdr in ['To','Cc','Bcc']:
            if self.msg.has_key(hdr):
                self.recipients.extend(self.msg.getaddrlist(hdr))
        self.recipients = map(lambda x:x[1], self.recipients)
        self.wikiRecipients = filter(lambda x:re.search(RECOGNIZEDADDREXP,x),
                                     self.recipients)
        self.wikiRecipients = map(lambda x:string.split(x,'@'),
                                  self.wikiRecipients)
        if self.isMime():
            self.body = self.firstMimePartMatching('text/plain')
        else:
            self.body = self.msg.fp.read()
        # strip trailing newlines that seem to get added in transit
        self.body = re.sub(r'(?s)\n+$',r'\n',self.body)

    def isMime(self):
        return re.search(r'Content-Type:.*multipart',self.raw)

    # based on python manual example
    def firstMimePartMatching(self,mimetype):
        msgtype = self.msg.gettype()
        params = self.msg.getplist()
        data = StringIO.StringIO()
        if msgtype[:10] == "multipart/":
            file = multifile.MultiFile(self.stream)
            file.push(self.msg.getparam("boundary"))
            while file.next():
                submsg = mimetools.Message(file)
                try:
                    data = StringIO.StringIO()
                    mimetools.decode(file, data, submsg.getencoding())
                except ValueError:
                    continue
                if submsg.gettype() == mimetype:
                    break
            file.pop()
        return data.getvalue()

    
def findVirtualHostFor(self, msg):
    """
    If we have one of the special recipients listed above along with a
    matching virtual host, return the virtual host's folder; otherwise None.
    Also save the username part of the matched recipient for later.
    """
    folder = None
    vurt = getattr(self,'vurt',None)
    if vurt and msg.wikiRecipients:
        for alias, host in msg.wikiRecipients:
            dict = getattr(vurt, 'domain_path', None)
            if dict:
                real_host=None
                l_host = string.lower(host)
                if vurt.domain_path.has_key(l_host):
                    real_host=l_host
                else:
                    for hostname in vurt.domain_path.keys():
                        if hostname[:2] == '*.':
                            if string.find(l_host, hostname[2:]) != -1:
                                real_host = hostname
                if real_host:
                    path = vurt.domain_path[real_host]
                    folder = self.getPhysicalRoot().unrestrictedTraverse(path)
                    msg.alias = alias
                    break
    return folder

def findDestinationFor(self, msg, pagenameexp, defaultpage,
                       checkrecipient, checksubject):
    """
    figure out the destination wiki folder and page for an incoming
    message. See rules above.
    """
    if self.meta_type == 'ZWiki Page':
        folder, page, creating = self.aq_parent, self, 0
    else:
        folder = findVirtualHostFor(self,msg) or self
        pagename = None
        #if checkrecipient:
        #    pagename = re.search(pagenameexp, string.join(recipients))
        #    if pagename:
        #        pagename = pagename.group(1)
        # or in the subject
        if (not pagename) and checksubject:
            pagename = re.search(pagenameexp,msg.subject)
            if pagename:
                pagename = pagename.group(1)
        if not pagename:
            pagename = getattr(folder,'default_page',defaultpage)

        if hasattr(folder,pagename):
            page = getattr(folder,pagename)
            creating = 0
        else:
            page = getattr(folder,defaultpage)
            creating = 1

    return (folder,page,creating)

def mailin(self, msg, pagenameexp=WIKINAMEEXP, defaultpage=DEFAULTPAGE,
           separator='\n\n', checkrecipient=0, checksubject=1, trackerissue=0,
           subscribersonly=1):
    # mail loop protection
    # XXX should do a more specific X-BeenThere check
    if re.search(r'(?m)^X-Zwiki-Version: (.*)',msg):
        return
    msg = IncomingMessage(msg)
    folder,page,creating = findDestinationFor(self,msg,
                                              pagenameexp,defaultpage,
                                              checkrecipient,checksubject)
    if (subscribersonly and
        not getattr(folder,'posting_policy',None) == 'open'):
        if not len(page.allSubscriptionsFor(msg.senderEmail)):
            return '\nSorry, you must be a subscriber to send mail to this wiki.\n\n\n'
        
    # create a tracker issue ?
    # temp hack if you don't have a separate mail address - look for
    # "(tracker)" in the recipients
    if (trackerissue or
        re.search(r'tracker$',getattr(msg,'alias','')) or
        re.search(r'\(tracker\)', string.join(msg.recipients))):
        subject = msg.subject or 'no subject'
        self.REQUEST.set('newtitle', subject)
        # italicize quoted text in replies (as in comment(), but remember DRY)
        body = re.sub(r'(?m)^>(.*)',r'<br>><i>\1</i>',msg.body)
        self.REQUEST.set('newtext', body)
        self.REQUEST.set('submitted', 1)
        page.aq_parent.AddIssue(REQUEST=self.REQUEST)
        return 

    # create a page ?
    if creating:
        page.comment(use_heading=1,text=page.getId(),username=msg.sender,
                     note=' (new page)',REQUEST=self.REQUEST)
        # zwiki.org: REQUEST has no URL2, create fails. Leave out for now
        page.create(pagename,text='') #,REQUEST=self.REQUEST) 
        page = getattr(self,pagename)

    # add comment
    # use the time of sending, or
    #page.comment(use_heading=1, text=text, username=sender, time=date, note=' (via mail)')
    # use the time of posting to the wiki - see how this works out
    page.comment(use_heading=1, text=msg.body, username=msg.sender, note=' (via mail)',
                 REQUEST=self.REQUEST)
