# SubscriberList mixin

import string, re
from types import *


class SubscriberList:                 
    """
    RESPONSIBILITIES
    - store & help manage a list of email subscribers to a ZWikiPage
    - expose the ZWikiPage's subscribers as a ZMI property
    - also, provide the same services for the page's parent folder
      (ugly but convenient)

    COLLABORATORS
    ZWikiPage ?

    NB for convenience a ZWikiPage has this mixed-in;
    for a Folder we could install a manageable subclass of this as a
    subobject.  See if this makes sense.

    Bah too many complications. For now, overload the methods below to
    also manage a property in the parent folder. This is ugly and
    complicates testing but it's better than the alternatives I've thought
    of.
    """
    subscriber_list = []
    _properties=(
        {'id':'subscriber_list', 'type': 'lines', 'mode': 'w'},
        )

    ######################################################################
    # private methods

    def _getSubscribers(self, parent=0):
        """
        Get this page's email subscriber list. 
        With parent arg, manage the parent folder's subscriber list instead.
        """
        self.doSubscriberListFixups()
        if parent:
            if hasattr(self.aq_parent,'subscriber_list'):
                return self.aq_parent.subscriber_list
            else:
                return []
        else:
            return self.subscriber_list

    def _setSubscribers(self, subscriberlist, parent=0):
        """
        Set this page's email subscriber list. 
        With parent arg, manage the parent folder's subscriber list instead.
        """
        self.doSubscriberListFixups()
        if parent:
            self.aq_parent.subscriber_list = subscriberlist
        else:
            self.subscriber_list = subscriberlist

    def _resetSubscribers(self, parent=0):
        """
        Clear this page's subscriber list.
        With parent arg, manage the parent folder's subscriber list instead.
        """
        self._setSubscribers([],parent)

    ######################################################################
    # page subscription methods

    def subscriberList(self, parent=0):
        """
        Return a list of this page's subscribers
        With parent arg, manage the parent folder's subscriber list instead.
        """
        return self._getSubscribers(parent)

    def subscriberCount(self, parent=0):
        """
        Return the number of email addresses currently subscribed to
        this page
        With parent arg, manage the parent folder's subscriber list instead.
        """
        return len(self.subscriberList(parent))

    def isSubscriber(self,email,parent=0):
        """
        Is email currently subscribed to this page ?
        With parent arg, manage the parent folder's subscriber list instead.

        >>> # use this so we can test folder subscription
        >>> from Products.ZWiki.tests.support import MockZWikiPageInFolder
        >>> sl = MockZWikiPageInFolder(__name__='TestPage')
        >>> sl._setSubscribers(['a','b'])
        >>> sl.isSubscriber('a')
        1
        >>> sl.isSubscriber('c')
        0
        >>> sl.isSubscriber('')
        0
        >>> sl.isSubscriber(' ')
        0
        
        """
        if email:
            for s in self.subscriberList(parent):
                if string.lower(s) == string.lower(email):
                    return 1
        return 0
               
    def subscribe(self, email, REQUEST=None, parent=0):
        """
        Add email as a subscriber to this page.
        With parent arg, manage the parent folder's subscriber list instead.

        >>> from Products.ZWiki.tests.support import MockZWikiPageInFolder
        >>> sl = MockZWikiPageInFolder(__name__='TestPage')
        >>> sl._setSubscribers(['spamandeggs'])
        >>> sl.subscriberList()
        ['spamandeggs']
        >>> sl._resetSubscribers()
        >>> sl.subscriberList()
        []
        >>> sl.subscriberCount()
        0
        >>> sl.subscribe('1@test.com')
        >>> sl.subscriberList()
        ['1@test.com']
        >>> sl.subscriberCount()
        1
        >>> sl.isSubscriber('1@test.com')
        1
        >>> sl.isSubscriber('2@test.com')
        0
        >>> sl.subscribe('2@test.com')
        >>> sl.subscriberList()
        ['1@test.com', '2@test.com']
        >>> sl.subscriberCount()
        2
        >>> sl.isSubscriber('2@test.com')
        1
        >>> sl.subscribe('1@test.com')
        >>> sl.subscriberList()
        ['1@test.com', '2@test.com']
        >>> sl.unsubscribe('1@test.com')
        >>> sl.subscriberList()
        ['2@test.com']
        >>> sl.unsubscribe('1@test.com')
        >>> sl.subscriberList()
        ['2@test.com']
        >>> sl.unsubscribe('2@TesT.cOm')
        >>> sl.subscriberList()
        []
        """
        if email:
            if not self.isSubscriber(email,parent):
                subs = self._getSubscribers(parent)
                subs.append(email)
                self._setSubscribers(subs,parent)

        if REQUEST:
            REQUEST.RESPONSE.redirect(
                REQUEST['URL1']+'/subscribeform?email='+email)

    def unsubscribe(self, email, REQUEST=None, parent=0):
        """
        Remove an email address from this page's mail subscriber list.
        With parent arg, manage the parent folder's subscriber list instead.
        """
        if self.isSubscriber(email,parent):
            sl = self.subscriberList(parent)
            for s in sl:
                if string.lower(s) == string.lower(email):
                    sl.remove(s)
            self._setSubscribers(sl,parent)

        if REQUEST:
            REQUEST.RESPONSE.redirect(
                REQUEST['URL1']+'/subscribeform?email='+email)

    ######################################################################
    # folder subscription methods

    def wikiSubscriberList(self):
        """whole-wiki version of subscriberList"""
        return self.subscriberList(parent=1)

    def wikiSubscriberCount(self):
        """whole-wiki version of subscriberCount"""
        return self.subscriberCount(parent=1)

    def isWikiSubscriber(self,email):
        """whole-wiki version of isSubscriber"""
        return self.isSubscriber(email,parent=1)

    def wikiSubscribe(self, email, REQUEST=None):
        """whole-wiki version of subscribe"""
        return self.subscribe(email,REQUEST,parent=1)

    def wikiUnsubscribe(self, email, REQUEST=None):
        """whole-wiki version of unsubscribe"""
        return self.unsubscribe(email,REQUEST,parent=1)

    ######################################################################
    # misc methods

    def allSubscribers(self):
        """
        List all subscribers to this page or the wiki.

        >>> from Products.ZWiki.tests.support import MockZWikiPageInFolder
        >>> sl = MockZWikiPageInFolder(__name__='TestPage')
        >>> 
        >>> sl._resetSubscribers()
        >>> sl._resetSubscribers(parent=1)
        >>> sl.allSubscribers()
        []
        """
        subs = []
        subs.extend(self.subscriberList()) # copy, don't reference
        for s in self.wikiSubscriberList():
            if not (s in subs): subs.append(s)
        return subs

    def allSubscriptionsFor(self, email):
        """
        Return a list of all pages to which email is subscribed
        ('whole_wiki' indicates a wiki subscription).

        >>> from Products.ZWiki.tests.support import MockZWikiPageInFolder
        >>> p = MockZWikiPageInFolder(__name__='TestPage')
        >>> p._resetSubscribers()
        >>> p._resetSubscribers(parent=1)
        >>> p.subscribe('a@a.a')
        >>> p.subscribe('b@b.b')
        >>> p.wikiSubscribe('b@b.b')
        >>> p.allSubscriptionsFor('a@a.a')
        ['TestPage']
        >>> p.allSubscriptionsFor('b@b.b')
        ['whole_wiki', 'TestPage']
        >>> p.allSubscriptionsFor('c@c.c')
        []
        """
        # simple-minded implementation
        subscriptions = []
        if self.isWikiSubscriber(email):
            subscriptions.append('whole_wiki')
        for id, page in self.aq_parent.objectItems(spec='ZWiki Page'):
            if page.isSubscriber(email):
                subscriptions.append(id)
        return subscriptions

    def otherPageSubscriptionsFor(self, email):
        """
        Ack, this was too hard in DTML. Return a list of all pages to
        which email is subscribed, excluding the current page and
        'whole_wiki'.

        >>> from Products.ZWiki.tests.support import MockZWikiPageInFolder
        >>> thispage = MockZWikiPageInFolder(__name__='ThisPage')
        >>> thispage.create('ThatPage')
        >>> thatpage = thispage.aq_parent.ThatPage
        >>> thispage.subscribe('me')
        >>> thatpage.subscribe('me')
        >>> thatpage.wikiSubscribe('me')
        >>> thispage.otherPageSubscriptionsFor('me')
        ['ThatPage']
        """
        #python2.2ism
        #return filter(lambda x: not (x == self.id() or x == 'whole_wiki'),
        #              self.allSubscriptionsFor(email))
        subs = self.allSubscriptionsFor(email)
        thispage = self.id()
        if thispage in subs: subs.remove(thispage)
        if 'whole_wiki' in subs: subs.remove('whole_wiki')
        return subs

    def doSubscriberListFixups(self):
        """
        convert old subscribers property to new subscriber_list property
        as needed, for page and parent folder
        """
        ob = self.aq_parent
        if (not hasattr(ob, 'subscriber_list')):
            ob.subscriber_list = []
            ob._properties=ob._properties + \
                 ({'id':'subscriber_list', 'type': 'lines', 'mode': 'w'},)
        if (hasattr(ob, 'subscribers') and
            type(ob.subscribers) is StringType):
            if ob.subscribers:
                ob.subscriber_list = string.split(re.sub(r'[ \t]+',
                                                         r'',
                                                         ob.subscribers),
                                                  ',')
            try: del ob.subscribers
            except KeyError: pass
        ob = self
        if (hasattr(ob, 'subscribers') and
            type(ob.subscribers) is StringType  and
            ob.subscribers):
            ob.subscriber_list = string.split(re.sub(r'[ \t]+',
                                                     r'',
                                                     ob.subscribers),
                                              ',')
            try: del ob.subscribers
            except KeyError: pass
            self.last_editor_ip = ''
            self.last_editor = 'subscriberListUpgrade'
